// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "MeshParametersManager.h"

// standart C/C++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

namespace MAd {

  // -------------------------------------------------------------------
  MeshParametersManager::MeshParametersManager():
    lowerLengthSqBound(-1.), upperLengthSqBound(-1.), 
    sliverTetQualityBound(-1.), sliverTriQualityBound(-1.), 
    sliverPermInESplit(false), sliverPermInECollapse(false),
    noSwapQuality(-1.),swapMinImproveRatio(-1.)
  {}

  // -------------------------------------------------------------------
  void MeshParametersManager::initialize()
  {
  }

  // -------------------------------------------------------------------
  void MeshParametersManager::finalize()
  {
  }

  // -------------------------------------------------------------------
  // -------------------------------------------------------------------

  void MeshParametersManager::setLowerLengthSqBound(double lSq)
  {
    lowerLengthSqBound = lSq;
  }

  // -------------------------------------------------------------------

  void MeshParametersManager::setUpperLengthSqBound(double lSq)
  {
    upperLengthSqBound = lSq;
  }

  // -------------------------------------------------------------------
  double MeshParametersManager::getSliverBound(int dim) const
  {
    if (dim==3) return getSliverTetBound();
    return getSliverTriBound();
  }

  // -------------------------------------------------------------------
  double MeshParametersManager::getSliverBound(const pMesh mesh) const
  {
    return getSliverBound(M_dim(mesh));
  }

  // -------------------------------------------------------------------

  void MeshParametersManager::setSliverTriBound(double bound)
  {
    sliverTriQualityBound = bound;
  }

  // -------------------------------------------------------------------

  void MeshParametersManager::setSliverTetBound(double bound)
  {
    sliverTetQualityBound = bound;
  }

  // -------------------------------------------------------------------
  // -------------------------------------------------------------------

  void MeshParametersManager::setSliverPermissionInESplit(bool perm)
  {
    sliverPermInESplit = perm;
  }

  // -------------------------------------------------------------------

  void MeshParametersManager::setSliverPermissionInECollapse(bool perm)
  {
    sliverPermInECollapse = perm;
  }

  // -------------------------------------------------------------------
  // -------------------------------------------------------------------

  void MeshParametersManager::diagnostics() const
  {
    cout << "   *** Parameters for the mesh adaptation ***   \n";

    cout << "\n";

    cout << "- Bounds of the edge length squared: \n";
    cout << "\t Lower: " << lowerLengthSqBound << "\n";
    cout << "\t Upper: " << upperLengthSqBound << "\n";

    cout << "\n";

    cout << "- Quality bound for the sliver triangles: " 
         << sliverTriQualityBound << "\n";

    cout << "- Quality bound for the sliver tetrahedra: " 
         << sliverTetQualityBound << "\n";

    cout << "\n";

    cout << "Edge splits can create slivers: ";
    if (sliverPermInESplit) cout << "yes";
    else                    cout << "no";
    cout << "Edge collapses can create slivers: ";
    if (sliverPermInECollapse) cout << "yes";
    else                       cout << "no";
    cout << "\n";

    cout << "\n";
  }

  // -------------------------------------------------------------------

}
