// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "MAdOperatorBase.h"

namespace MAd {

  // -------------------------------------------------------------------
  bool MAdOperatorBase::evaluate(double * worstShape)
  {
    results->reset();
    *worstShape = mqm.getElementEvaluator()->worstShapeEver();

    History& history = HistorySgl::instance();

    // --- check 1: constraints ---
    if ( !checkConstraints() ) {
      history.add((int)type(),OP_CHECKCONSTRAINTS,0);
      return false;
    } else {
      history.add((int)type(),OP_CHECKCONSTRAINTS,1);
    }

    // --- check 2: geometric model ---
    if ( !checkGeometry() ) {
      history.add((int)type(),OP_CHECKGEOMETRY,0);
      return false;
    } else {
      history.add((int)type(),OP_CHECKGEOMETRY,1);
    }

    // --- check 3: elements validity and shapes ---
    if ( !evaluateShapes() ) {
      history.add((int)type(),OP_CHECKSHAPES,0);
      return false;
    } else {
      history.add((int)type(),OP_CHECKSHAPES,1);
    }

    // --- edge lengths ---
    evaluateLengths();
  
    *worstShape = results->getWorstShape();

    return true;
  }

  // -------------------------------------------------------------------

}
