// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_SLIVERFACEHANDLER
#define _H_SLIVERFACEHANDLER

#include "MAdOperatorBase.h"

#include <string>

namespace MAd {

  // -------------------------------------------------------------------
  class sliverFaceHandler {

  public:

    sliverFaceHandler(pMesh, pSField);
    ~sliverFaceHandler() {}

  public:

    void removeSliverFaces(int * nbIn=NULL, int * nbOut=NULL);

    void setSizeField(pSField);
    void collapseOnBoundary(bool, double);

    void enableReport(std::string);

  private:

    int findOperation(pFace, pMAdOperator *);

    bool F_isSliver(pFace);
    bool F_isSliver(pFace, double *);

    void reportSliver(pFace);

  private:

    pMesh mesh;
    pSField sizeField;
    MeshQualityManager& mqm;

    bool collapseOnBdry;
    double dATol;

    bool   reportFailures;
    int    reportId;
    std::string reportPrefix;

  };

  // -------------------------------------------------------------------
  inline sliverFaceHandler::sliverFaceHandler(pMesh m, pSField sf):
    mesh(m),sizeField(sf), mqm(MeshQualityManagerSgl::instance()), 
    collapseOnBdry(false), dATol(MAdTOL),
    reportFailures(false),reportId(0),reportPrefix("")
  {}

  // -------------------------------------------------------------------
  inline void sliverFaceHandler::setSizeField(pSField sf)
  {
    sizeField = sf;
  }

  // -------------------------------------------------------------------
  inline void sliverFaceHandler::collapseOnBoundary(bool cob, double tolerance)
  {
    collapseOnBdry = cob;
    dATol = tolerance;
  }

  // -------------------------------------------------------------------
  inline void sliverFaceHandler::enableReport(std::string prefix)
  {
    reportFailures = true; 
    reportPrefix   = prefix;
  };

  // -------------------------------------------------------------------

}

#endif

