// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_ELEMENTEVALUATORBASE
#define _H_ELEMENTEVALUATORBASE

#include "SizeFieldBase.h"
#include "MSops.h"

#include <string>

namespace MAd {

  // -------------------------------------------------------------------
  enum evaluationType {
    UNKNOWNEVALTYPE,
    MEANRATIO
  };

  // -------------------------------------------------------------------
  class elementEvaluatorBase {

  public:

    elementEvaluatorBase(const pSField f): sizeField(f) { setDefaultAcptValue(); }
    virtual ~elementEvaluatorBase() {};
  
    void setSizeField(pSField sf) {sizeField = sf;}

    virtual int R_shape(pRegion r, double * result) const = 0;
    virtual int F_shape(pFace f, double normal[3], double * result) const = 0;
    virtual int XYZ_R_shape(double[4][3], pMSize[4], double * result) const = 0;
    virtual int XYZ_F_shape(double[3][3], pMSize[3], double normal[3], double * result) const = 0;

    virtual inline double worstShapeEver() const {return 0.;}
    virtual inline double bestShapeEver()  const {return 1.;}
    virtual inline double whatsWorst(double shp1, double shp2) const {return std::min(shp1,shp2);}

    virtual evaluationType getType () const = 0;
    virtual std::string getName () const = 0;

    double getAcceptableValue() { return acptValue; }
    void setAcptValue(double t) { acptValue = t; }
    void setDefaultAcptValue() { acptValue = 1.0e-6; }

  protected:
 
    pSField sizeField;
    double acptValue;
  };

  // -------------------------------------------------------------------

}

#endif
