// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_MADELASTICITYOP
#define _H_MADELASTICITYOP

#include "MAdMatrix.h"
#include "VertexMoveOp.h"

#include <map>
#include <set>

namespace MAd {

  // -------------------------------------------------------------------
  class MAdElasticityOp {

  public:

    MAdElasticityOp(pMesh m);
    MAdElasticityOp(const MAdElasticityOp&);
    virtual ~MAdElasticityOp() { localIds.clear(); }

  public:

    int meshDim() const { return dim; }

    void setMaterials(double _E, double _nu);
    void setStiffnessAlterationCoef(double _chi);

    void setHomogDirichletBC();
    void addDirichlet(pVertex pv, smallVector& dxyz);
    void delDirichlet(pVertex pv);

    int run();

  private:

    void generateVertexIds();
    void collectBCVertices(std::set<pVertex> * bcVerts);
  
    void element3DMatrix (pRegion pr, smallMatrix& m) const;

  private:

    pMesh mesh;
    int dim;

    double E, nu;

    // exponent of the alteration of stiffness based on elements size:
    // chi=0 => no alteration
    // chi increases => smaller elements stiffer
    double chi;

    std::map<pVertex,int> localIds;

    std::map<pVertex,smallVector> dirichlet;

  };

  // -------------------------------------------------------------------

}

#endif
