// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_ANISOMESHSIZE
#define _H_ANISOMESHSIZE

#include "MeshSizeBase.h"
#include "MAdDefines.h"

namespace MAd {

  // -------------------------------------------------------------------
  // This class stores an anisotropic mesh size (prescribed edge length)
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  class AnisoMeshSize : public MeshSizeBase {

  public:
  
    AnisoMeshSize();
    AnisoMeshSize(double e[3][3], double _h[3]);
    AnisoMeshSize(double _h[3], double eDiag[3], double eOffDiag[3]);
    AnisoMeshSize(double _h);
    AnisoMeshSize(AnisoMeshSize * _ams);
    ~AnisoMeshSize() {};

  private:

    void processMetric(const double dirs[3][3], const double hs[3]);

  public:

    MeshSizeType getType() const;

    void   toMetricSpace(const double[3], double[3]) const;

    void   intersect(const pMSize pMS0, const pMSize pMS1);

    void   interpolate(const pMSize pMS0, const pMSize pMS1, double param);

    void   direction(int i, double dir[3]) const;
    double size(int i=0) const { return h[i]; }
    double lengthSqInDir(double dir[3]) const ;
    double getMeanLength() const { return MAdTHIRD * ( h[0] + h[1] + h[2] ); }

    void   scale(int, double);
    void   scale(double);

    void   print(std::string name="") const;

  private:

    void computeMetric(double metr[3][3]) const;
  
  private:

    // GCREMARK: we could use half of this space.
    double h[3];
    double e[3][3]; // one row = one eigenvector of the metric
  };

  // -------------------------------------------------------------------

}

#endif
