// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_PARAMETERS
#define _H_PARAMETERS

#include "AdaptInterface.h"

#include <string.h>
using std::string;
#include <list>
#include <set>
#include <vector>
using std::vector;

// ----------------------------------------------------------------------
enum KinematicType {
  KT_DISPLACEMENT,
  KT_VELOCITY_TRANSLATION,
  KT_VELOCITY_FULLRANDOM,
  KT_VELOCITY_RANDOMTRANSLATION,
  KT_VELOCITY_ISOMETRY,
  KT_VELOCITY_ROTATION,
  KT_VELOCITY_SHEAR_YZ,
  KT_VELOCITY_SHEAR_ZX,
  KT_VELOCITY_SHEAR_XY,
  KT_VELOCITY_EXPRESSION
};

// ----------------------------------------------------------------------
enum OrientationType {
  ORT_ISOTROPIC,
  ORT_ANISOTROPIC
};

// ----------------------------------------------------------------------
// ----------------------------------------------------------------------
struct LocalSFDef {
  double radius, sizeIn, sizeOut;
};

// ----------------------------------------------------------------------
struct AnalSFDef {
  OrientationType orientation;
  string isoSize;
  vector<string> anisoSize;
  vector<string> anisoDir0, anisoDir1, anisoDir2;
};

// ----------------------------------------------------------------------
struct ObjectDef {

  string name;

  // geometry
  int geomLevel; // 0=point,1=line,2=surface,3=region
  std::set<int> geomTags;

  // kinematics
  KinematicType kinType;
  vector<string> kinExpression;
  double center[3], velocity[3];

  // size fields
  std::list<LocalSFDef> sizes;
};

// ----------------------------------------------------------------------
struct MoveParameters {
  
  // global input specifications
  // ---------------------------

  string meshFileName;
  string geoFileName;
  
  // global output parameters
  // ------------------------

  int outFrequency;
  string outType; // "Msh", "Pos" or "MshAndPos"
  string outPrefix;

  // debugging parmeters
  // --------------------

  bool debugMode;
  bool openJournal;
  string referenceJournal;
  bool sliverReports;
  bool testSliverOperators;

  // Mesh adaptation
  // ----------------

  int maxInnerIter;
  double lowerLength, upperLength;
  double noSwapQuality;
  double swapMinImproveRatio;
  double sliverQuality;
  bool collapseOnBoundary;
  double boundaryTolerance;
  bool trackGeometry;
  
  // Moving objects
  // --------------

  std::vector<ObjectDef> objects;

  // Size Field parameters
  // ---------------------

  std::list<AnalSFDef> sizes;
  
  // Node motion
  // -----------

  string nodeMotionType; // ForceBoundaries, ElasticAnalogy
  double stiffnessAlteration;

  // Time parameters
  // ---------------

  int maxNbTimeSteps;
  double timeStep;
  double finalTime;

};

// ----------------------------------------------------------------------
// this function set the parameters to the values of a particular test case
extern void setCurrentParameters(MoveParameters *);

// ----------------------------------------------------------------------
void setDefaultParameters(MoveParameters * params)
{
  params->meshFileName = "";
  params->geoFileName = "";

  params->outFrequency = 1;
  params->outType = "MshAndPos";
  params->outPrefix  = "result/";

  params->debugMode = false;
  params->openJournal = false;
  params->referenceJournal = "";
  params->sliverReports = false;
  params->testSliverOperators = false;

  params->maxInnerIter = 10;
  params->lowerLength = 0.5;
  params->upperLength = 1.6;
  params->noSwapQuality = 0.2;
  params->swapMinImproveRatio = 1.0;
  params->sliverQuality = 0.02;
  params->collapseOnBoundary = true;
  params->boundaryTolerance = 1.e-12;
  params->trackGeometry = false;

  params->objects.clear();

  params->sizes.clear();

  params->nodeMotionType = "ElasticAnalogy";
  params->stiffnessAlteration = 1.0;

  params->maxNbTimeSteps = 1000000;
  params->timeStep = 0.1;
  params->finalTime = -1.0;
}

// ----------------------------------------------------------------------
MoveParameters getParameters()
{
  MoveParameters params;
  setDefaultParameters(&params);
  setCurrentParameters(&params);

  return params;
}

// ----------------------------------------------------------------------

#endif
