// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#include "ModelInterface.h"

namespace MAd {

  // -------------------------------------------------------------------
  void GM_create(pGModel* model, std::string name)
  { 
    if (*model) delete (*model);
#ifdef _HAVE_GMSH_
    *model = new GmshModel(name);
#else
    *model = new NullModel(name);
#endif
  }

  // -------------------------------------------------------------------
  void GM_delete(pGModel model)
  { 
    if (model) { delete model; model=NULL; }
  }

  // -------------------------------------------------------------------
  int GM_readFromMSH(pGModel model, const std::string name)
  {
    return (model->readMSH(name));
  }

  // -------------------------------------------------------------------
  int GM_readFromGEO(pGModel model, const std::string name)
  {
    return (model->readGEO(name));
  }

  // -------------------------------------------------------------------
  pGEntity GM_entityByTag(pGModel model, int dim, int tag)
  {
    return model->getEntityByTag(dim,tag);
  }
  pGRegion GM_regionByTag(const pGModel model, int tag)
  {
    return model->getRegionByTag(tag);
  }
  pGFace GM_faceByTag(const pGModel model, int tag)
  {
    return model->getFaceByTag(tag);
  }
  pGEdge GM_edgeByTag(const pGModel model, int tag)
  {
    return model->getEdgeByTag(tag);
  }
  pGVertex GM_vertexByTag(const pGModel model, int tag)
  {
    return model->getVertexByTag(tag);
  }

  // -------------------------------------------------------------------
  int GM_numVertices(const pGModel model)
  {
    return model->getNumVertices();
  }
  int GM_numEdges(const pGModel model)
  {
    return model->getNumEdges();
  }
  int GM_numFaces(const pGModel model)
  {
    return model->getNumFaces();
  }
  int GM_numRegions(const pGModel model)
  {
    return model->getNumRegions();
  }

  // -------------------------------------------------------------------
  GRIter GM_regionIter(pGModel model)
  {
    return new GM_RegionIterator(model);
  }
  GFIter GM_faceIter(pGModel model)
  {
    return new GM_FaceIterator(model);
  }
  GEIter GM_edgeIter(pGModel model)
  {
    return new GM_EdgeIterator(model);
  }
  GVIter GM_vertexIter(pGModel model)
  {
    return new GM_VertexIterator(model);
  }

  pGRegion GRIter_next(GRIter iter) { return iter->next(); }
  pGFace   GFIter_next(GFIter iter) { return iter->next(); }
  pGEdge   GEIter_next(GEIter iter) { return iter->next(); }
  pGVertex GVIter_next(GVIter iter) { return iter->next(); }

  void GRIter_delete(GRIter iter) { delete (iter); }
  void GFIter_delete(GFIter iter) { delete (iter); }
  void GEIter_delete(GEIter iter) { delete (iter); }
  void GVIter_delete(GVIter iter) { delete (iter); }

  void GRIter_reset(GRIter iter) { iter->reset(); }
  void GFIter_reset(GFIter iter) { iter->reset(); }
  void GEIter_reset(GEIter iter) { iter->reset(); }
  void GVIter_reset(GVIter iter) { iter->reset(); }

  // -------------------------------------------------------------------
  int GEN_tag(const pGEntity ent)
  {
    return ent->tag();
  }

  // -------------------------------------------------------------------
  int GEN_type(const pGEntity ent)
  {
    return ent->dim();
  }

#ifdef _HAVE_GMSH_
  // -------------------------------------------------------------------
  std::list<pGEntity> GEN_closure(const pGEntity pGE)
  {
    std::list<pGEntity> theList;

    int type = GEN_type(pGE);
    switch (type) {
    case 0: break;
    case 1: {
      std::list<pGVertex> vList = GE_vertices((pGEdge)pGE);
      std::list<pGVertex>::const_iterator vIter = vList.begin();
      for (; vIter != vList.end(); vIter++) {
        theList.push_back(*vIter);
      }
      break;
    }
    case 2: {
      std::list<pGEdge> eList = GF_edges((pGFace)pGE);
      std::list<pGEdge>::const_iterator eIter = eList.begin();
      for (; eIter != eList.end(); eIter++) {
        theList.push_back(*eIter);
      }
      break;
    }
    case 3: {
      std::list<pGFace> fList = GR_faces((pGRegion)pGE);
      std::list<pGFace>::const_iterator fIter = fList.begin();
      for (; fIter != fList.end(); fIter++) {
        theList.push_back(*fIter);
      }
      break;
    }
    }

    return theList;
  }

  // -------------------------------------------------------------------
  std::list<pGFace> GR_faces(const pGRegion pGR)
  {
    return pGR->faces();
  }

  // -------------------------------------------------------------------
  int GF_numRegions(const pGFace f)
  {
    return f->numRegions();
  }

  // -------------------------------------------------------------------
  std::list<pGEdge> GF_edges(const pGFace pGF)
  {
    return pGF->edges();
  }

  // -------------------------------------------------------------------
  //! Computes the parameter location of xyz in the surface. 
  //! Returns false if xyz is not in the surface.
  bool GF_getParams(const pGFace pGF, const double xyz[3], 
                    double params[2])
  {
    //   SPoint2 sP = pGF->parFromPoint(SPoint3(xyz));
    //   params[0] = sP.x();
    //   params[1] = sP.y();

    throw;

    // a test should be done to check that xyz was on the surface

    return true;
  }

  // -------------------------------------------------------------------
  //! Computes the coordinates of the point on the surface closest to xyz
  void GF_closestPoint(const pGFace pGF, const double xyz[3],
                       const double initGuess[2], double xyzOnF[3])
  {
    //   GPoint gP = pGF->closestPoint( SPoint3(xyz), initGuess );
    //   xyzOnF[0] = gP.x();
    //   xyzOnF[1] = gP.y();
    //   xyzOnF[2] = gP.z();
    throw;
  }

  // -------------------------------------------------------------------
  void GF_xyz(const pGFace pGF, double u, double v, double xyz[3])
  {
    GPoint gP = pGF->point(u,v);
    xyz[0] = gP.x();
    xyz[1] = gP.y();
    xyz[2] = gP.z();
    printf("uv: %f %f, xyz: %f %f %f\n",u,v,xyz[0],xyz[1],xyz[2]);
  }

  // -------------------------------------------------------------------
  std::list<pGVertex> GE_vertices(const pGEdge pGE)
  {
    return pGE->vertices();
  }

  // -------------------------------------------------------------------
  //! Computes the coordinates of the point on the line closest to xyz
  void GE_closestPoint(const pGEdge pGE, const double xyz[3], 
                       double xyzOnE[3])
  {
    //   GPoint gP = pGE->closestPoint( SPoint3(xyz) );
    //   xyzOnE[0] = gP.x();
    //   xyzOnE[1] = gP.y();
    //   xyzOnE[2] = gP.z();
    throw;
  }

  // -------------------------------------------------------------------
  void GE_xyz(const pGEdge pGE, double u, double xyz[3])
  {
    GPoint gP = pGE->point(u);
    xyz[0] = gP.x();
    xyz[1] = gP.y();
    xyz[2] = gP.z();
  }

  // -------------------------------------------------------------------
  //! Given an edge which is a seam of the face, and a point of 
  //! the edge with parametric coordinate uOnE on the edge, 
  //! find the parametric coordinates of the point on the face.
  void GE_reparamOnFace(const pGEdge edge, const pGFace face, 
                        double uOnE, double uOnF[2])
  {
    //   assert( edge->isSeam(face) );

    SPoint2 pt = edge->reparamOnFace(face, uOnE, 0);
    uOnF[0] = pt.x();
    uOnF[1] = pt.y();
  }

  // -------------------------------------------------------------------
  //! Given a geometric vertex which is on the face, find the parametric 
  //! coordinates of the vertex on the face.
  void GV_reparamOnFace(const pGVertex vertex, const pGFace face, 
                        double uOnF[2])
  {
    SPoint2 pt = vertex->reparamOnFace(face, 0);
    uOnF[0] = pt.x();
    uOnF[1] = pt.y();
  }

  // -------------------------------------------------------------------
  //! Given a geometric vertex which is on the edge, find the parametric 
  //! coordinates of the vertex on the edge.
  void GV_reparamOnEdge(const pGVertex vertex, const pGEdge edge, 
                        double * uOnF)
  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,
                                "missing function in Gmsh");
    //   *uOnF = vertex->reparamOnEdge(edge, 0);
  }
#endif

  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  // PGList functions
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  PGList * PGList_new()
  {
    return new PGList();
  };

  // -------------------------------------------------------------------
  void PGList_delete(PGList * lst)
  {
    if (lst) delete lst;
    lst = NULL;
  };

  // -------------------------------------------------------------------
  void PGList_clear(PGList * lst)
  {
    lst->clear();
  };

  // -------------------------------------------------------------------
  PGList * PGList_appUnique(PGList * lst, pGEntity ent)
  {
    for ( unsigned int i=0; i<lst->entities.size(); i++ ) {
      if ( lst->entities[i] == ent ) return lst;
    }
    lst->entities.push_back(ent);
    return lst;
  };

  // -------------------------------------------------------------------
  PGList * PGList_appPGListUnique(PGList * lst, PGList * source)
  {
    for ( unsigned int iSrc=0; iSrc<source->entities.size(); iSrc++ ) {
      PGList_appUnique(lst,source->entities[iSrc]);
    }
    return lst;
  };

  // -------------------------------------------------------------------
  PGList * PGList_append(PGList * lst, pGEntity ent)
  {
    lst->entities.push_back(ent);
    return lst;
  };

  // -------------------------------------------------------------------
  int PGList_size(PGList * lst)
  {
    return lst->entities.size();
  };

  // -------------------------------------------------------------------
  pGEntity PGList_item(PGList * lst, int i)
  {
    return lst->entities[i];
  };

  // -------------------------------------------------------------------
  pGEntity PGList_next(PGList * lst, void **restart)
  {
    if( *(int*)(restart) >= (int)lst->entities.size() ) return NULL;
    return lst->entities[(*(int*)(restart))++];
  };

  // -------------------------------------------------------------------
  int PGList_inList(PGList * lst, pGEntity ent)
  {
    for ( unsigned int i=0; i<lst->entities.size(); i++ ) {
      if ( lst->entities[i] == ent ) return 1;
    }
    return 0;
  };

  // -------------------------------------------------------------------
  void PGList_remItem(PGList * lst, pGEntity ent)
  {
    std::vector<pGEntity>::iterator eIter = lst->entities.begin();
    for (; eIter != lst->entities.end() ; eIter++) {
      if ( *eIter == ent ) lst->entities.erase(eIter);
    }
  };

  // -------------------------------------------------------------------


}

