// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#ifndef _H_NULLENTITIES
#define _H_NULLENTITIES

#ifndef _HAVE_GMSH_

namespace MAd {

  // -------------------------------------------------------------------
  class NullGEntity {

  public:
    NullGEntity(): _tag(-1) {}
    NullGEntity(int tag): _tag(tag) {}
    NullGEntity(const NullGEntity& ge): _tag(ge._tag) {}
    virtual ~NullGEntity() {}

    virtual int dim() const { return -1; }
    virtual int tag() const { return _tag; }

  private:
    int _tag;
  };

  // -------------------------------------------------------------------
  class NullGEntityLessThan {
  public:
    bool operator()(const NullGEntity *ent1, const NullGEntity *ent2) const
    { return ent1->tag() < ent2->tag(); }
  };

  // -------------------------------------------------------------------
  class NullGRegion : public NullGEntity {

  public:
    NullGRegion() : NullGEntity() {}
    NullGRegion(int tag) : NullGEntity(tag) {}
    NullGRegion(const NullGRegion& gr) : NullGEntity(gr) {}
    virtual ~NullGRegion() {}
    int dim() const { return 3; }
  };

  // -------------------------------------------------------------------
  class NullGFace : public NullGEntity {

  public:
    NullGFace() : NullGEntity() {}
    NullGFace(int tag) : NullGEntity(tag) {}
    NullGFace(const NullGFace& gf) : NullGEntity(gf) {}
    virtual ~NullGFace() {}
    int dim() const { return 2; }
  };

  // -------------------------------------------------------------------
  class NullGEdge : public NullGEntity {

  public:
    NullGEdge() : NullGEntity() {}
    NullGEdge(int tag) : NullGEntity(tag) {}
    NullGEdge(const NullGEdge& ge) : NullGEntity(ge) {}
    virtual ~NullGEdge() {}
    int dim() const { return 1; }
  };

  // -------------------------------------------------------------------
  class NullGVertex : public NullGEntity {

  public:
    NullGVertex() : NullGEntity() {}
    NullGVertex(int tag) : NullGEntity(tag) {}
    NullGVertex(const NullGVertex& gv) : NullGEntity(gv) {}
    virtual ~NullGVertex() {}
    int dim() const { return 0; }
  };

  // -------------------------------------------------------------------
  typedef class NullGEntity MAdGEntity;
  typedef class NullGEntityLessThan MAdGEntityLessThan;
  typedef class NullGRegion MAdGRegion;
  typedef class NullGFace   MAdGFace;
  typedef class NullGEdge   MAdGEdge;
  typedef class NullGVertex MAdGVertex;

  // -------------------------------------------------------------------
}

#endif

#endif

